@@ -17,6 +17,7 @@
 use crate::NetworkError;
 
 use arc_swap::ArcSwap;
+use serde::{Deserialize, Serialize};
 use std::{
     net::SocketAddr,
     sync::Arc,
@@ -24,23 +25,35 @@ use std::{
     {self},
 };
 
+#[derive(Copy, Clone, Debug, Eq, PartialEq, Serialize, Deserialize)]
+pub enum NodeType {
+    /// The default "regular" node.
+    Client,
+    /// Crawls the network to track the known network.
+    Crawler,
+    /// Used for peer discovery, they are the entry point into the network.
+    Beacon,
+    /// Used for initial sync after joining the network.
+    ///
+    /// Sync provider addresses are shared by the beacons.
+    SyncProvider,
+}
+
 /// A core data structure containing the pre-configured parameters for the node.
 pub struct Config {
     /// The desired numeric ID of the node.
     pub node_id: Option<u64>,
+    pub node_type: NodeType,
     /// The pre-configured desired address of this node.
     pub desired_address: SocketAddr,
     /// The minimum number of peers required to maintain connections with.
     minimum_number_of_connected_peers: u16,
     /// The maximum number of peers permitted to maintain connections with.
     maximum_number_of_connected_peers: u16,
-    /// The default bootnodes of the network.
-    pub bootnodes: ArcSwap<Vec<SocketAddr>>,
-    /// If `true`, initializes this node as a bootnode and forgoes connecting
-    /// to the default bootnodes or saved peers in the peer book.
-    is_bootnode: bool,
-    /// If `true`, initializes this node as a crawler.
-    is_crawler: bool,
+    /// The default peer discovery nodes of the network.
+    pub beacons: ArcSwap<Vec<SocketAddr>>,
+    /// The default sync provider nodes of the network.
+    sync_providers: ArcSwap<Vec<SocketAddr>>,
     /// The interval between each peer sync.
     peer_sync_interval: Duration,
 }
@@ -50,56 +63,47 @@ impl Config {
     #[allow(clippy::too_many_arguments)]
     pub fn new(
         node_id: Option<u64>,
+        node_type: NodeType,
         desired_address: SocketAddr,
         minimum_number_of_connected_peers: u16,
         maximum_number_of_connected_peers: u16,
-        bootnodes_addresses: Vec<String>,
-        is_bootnode: bool,
-        is_crawler: bool,
+        beacon_addresses: Vec<String>,
+        sync_provider_addresses: Vec<String>,
         peer_sync_interval: Duration,
     ) -> Result<Self, NetworkError> {
-        // Convert the given bootnodes into socket addresses.
-        let mut bootnodes = Vec::with_capacity(bootnodes_addresses.len());
-        for bootnode_address in bootnodes_addresses.iter() {
-            if let Ok(bootnode) = bootnode_address.parse::<SocketAddr>() {
-                bootnodes.push(bootnode);
-            }
-        }
+        // Convert the given seeded nodes into socket addresses.
+        let beacons: Vec<SocketAddr> = beacon_addresses
+            .into_iter()
+            .filter_map(|addr| addr.parse().ok())
+            .collect();
+
+        let sync_providers: Vec<SocketAddr> = sync_provider_addresses
+            .into_iter()
+            .filter_map(|addr| addr.parse().ok())
+            .collect();
 
         Ok(Self {
             node_id,
+            node_type,
             desired_address,
             minimum_number_of_connected_peers,
             maximum_number_of_connected_peers,
-            bootnodes: ArcSwap::new(Arc::new(bootnodes)),
-            is_bootnode,
-            is_crawler,
+            beacons: ArcSwap::new(Arc::new(beacons)),
+            sync_providers: ArcSwap::new(Arc::new(sync_providers)),
             peer_sync_interval,
         })
     }
 
-    /// Returns the default bootnodes of the network.
+    /// Returns the default peer discovery nodes of the network.
     #[inline]
-    pub fn bootnodes(&self) -> Arc<Vec<SocketAddr>> {
-        self.bootnodes.load_full()
+    pub fn beacons(&self) -> Arc<Vec<SocketAddr>> {
+        self.beacons.load_full()
     }
 
-    /// Returns `true` if this node is a bootnode. Otherwise, returns `false`.
+    /// Returns the default sync provider nodes of the network.
     #[inline]
-    pub fn is_bootnode(&self) -> bool {
-        self.is_bootnode
-    }
-
-    /// Returns `true` if this node is a crawler. Otherwise, returns `false`.
-    #[inline]
-    pub fn is_crawler(&self) -> bool {
-        self.is_crawler
-    }
-
-    /// Returns `true` if this node is a plain node. Otherwise, returns `false`.
-    #[inline]
-    pub fn is_regular_node(&self) -> bool {
-        !(self.is_bootnode() || self.is_crawler())
+    pub fn sync_providers(&self) -> Arc<Vec<SocketAddr>> {
+        self.sync_providers.load_full()
     }
 
     /// Returns the minimum number of peers this node maintains a connection with.
