@@ -15,7 +15,7 @@
 // along with the snarkOS library. If not, see <https://www.gnu.org/licenses/>.
 
 use crate::*;
-use snarkos_metrics::{self as metrics, misc, wrapped_mpsc};
+use snarkos_metrics::{self as metrics, wrapped_mpsc};
 
 use anyhow::*;
 use chrono::{DateTime, Utc};
@@ -124,9 +124,9 @@ impl Node {
             state: Default::default(),
             local_addr: Default::default(),
             config,
-            storage,
+            storage: storage.clone(),
             inbound_cache: Default::default(),
-            peer_book: PeerBook::spawn(),
+            peer_book: PeerBook::spawn(storage),
             sync: Default::default(),
             known_network: Default::default(),
             launched: Utc::now(),
@@ -183,6 +183,16 @@ impl Node {
     pub async fn start_services(&self) {
         let node_clone: Node = self.clone();
         let peer_sync_interval = self.config.peer_sync_interval();
+        match self.storage.fetch_peers().await {
+            Err(e) => {
+                error!("failed to fetch peers from storage: {:?}", e);
+            }
+            Ok(peers) => {
+                for peer in peers {
+                    self.peer_book.add_peer(peer.address, Some(&peer)).await;
+                }
+            }
+        }
         let peering_task = task::spawn(async move {
             loop {
                 info!("Updating peers");
@@ -234,6 +244,9 @@ impl Node {
                             if let Err(e) = node_clone.run_sync().await {
                                 error!("failed sync process: {:?}", e);
                             }
+                            if let Err(e) = node_clone.expect_sync().consensus.fast_forward().await {
+                                error!("failed to initiate fast forward for sync: {:?}", e);
+                            };
                             node_clone.finished_syncing_blocks();
                         }
                     }
@@ -248,9 +261,7 @@ impl Node {
             #[cfg(not(feature = "test"))]
             sleep(Duration::from_secs(5)).await;
 
-            // FIXME: sync providers are set as beacons in the configuration during the rollout.
-            // let sync_providers = self.config.sync_providers();
-            let sync_providers = self.config.beacons();
+            let sync_providers = self.config.sync_providers();
             let node_clone = self.clone();
             let mempool_sync_interval = node_clone.expect_sync().mempool_sync_interval();
             let sync_mempool_task = task::spawn(async move {
@@ -348,7 +359,7 @@ impl Node {
 
         // The node can already be at some non-zero height.
         if self.sync().is_some() {
-            metrics::gauge!(misc::BLOCK_HEIGHT, self.storage.canon().await?.block_height as f64);
+            metrics::gauge!(metrics::blocks::HEIGHT, self.storage.canon().await?.block_height as f64);
         }
 
         Ok(())
