@@ -16,7 +16,6 @@
 
 use std::time::Instant;
 
-use snarkos_storage::SerialBlock;
 use tokio::sync::oneshot;
 
 use snarkos_metrics::{self as metrics, wrapped_mpsc};
@@ -28,7 +27,6 @@ use super::network::PeerIOHandle;
 pub(super) enum PeerAction {
     Disconnect,
     Send(Payload, Option<Instant>),
-    SendBlock(Box<SerialBlock>, u32), // block, height
     Get(oneshot::Sender<Peer>),
     QualityJudgement,
     CancelSync,
@@ -62,13 +60,6 @@ impl PeerHandle {
         self.sender.send(PeerAction::Send(payload, time_received)).await.ok();
     }
 
-    pub async fn send_block(&self, payload: SerialBlock, height: u32) {
-        self.sender
-            .send(PeerAction::SendBlock(Box::new(payload), height))
-            .await
-            .ok();
-    }
-
     pub async fn cancel_sync(&self) {
         self.sender.send(PeerAction::CancelSync).await.ok();
     }
@@ -99,31 +90,20 @@ impl Peer {
     ) -> Result<PeerResponse, NetworkError> {
         match message {
             PeerAction::Disconnect => Ok(PeerResponse::Disconnect),
-            PeerAction::SendBlock(block, height) => {
-                // check if they sent us the block recently
-                let block = block.serialize();
-
-                if self.block_received_cache.contains(&block[..]) {
-                    metrics::increment_counter!(metrics::outbound::ALL_CACHE_HITS);
-                    return Ok(PeerResponse::None);
-                }
-
-                let message = Payload::Block(block, Some(height));
-
-                network.write_payload(&message).await.map_err(|e| {
-                    metrics::increment_counter!(metrics::outbound::ALL_FAILURES);
-                    e
-                })?;
-
-                metrics::increment_counter!(metrics::outbound::ALL_SUCCESSES);
-
-                debug!("Sent a '{}' message to {}", &message, self.address);
-                Ok(PeerResponse::None)
-            }
             PeerAction::Send(message, time_received) => {
-                if matches!(message, Payload::Ping(_)) {
-                    self.quality.expecting_pong = true;
-                    self.quality.last_ping_sent = Some(Instant::now());
+                match &message {
+                    Payload::Ping(_) => self.start_ping_measurement(),
+                    Payload::Block(block, _) => {
+                        if self.block_received_cache.contains(&block[..]) {
+                            metrics::increment_counter!(metrics::outbound::ALL_CACHE_HITS);
+                            return Ok(PeerResponse::None);
+                        }
+                        self.sync_state.blocks_sent_to += 1;
+                    }
+                    Payload::SyncBlock(..) => {
+                        self.sync_state.blocks_synced_to += 1;
+                    }
+                    _ => (),
                 }
 
                 network.write_payload(&message).await.map_err(|e| {
@@ -163,37 +143,16 @@ impl Peer {
                 }
             }
             PeerAction::CancelSync => {
-                if self.quality.remaining_sync_blocks > self.quality.total_sync_blocks / 2 {
-                    warn!(
-                        "Was expecting {} more sync blocks from {}",
-                        self.quality.remaining_sync_blocks, self.address,
-                    );
-                    self.quality.remaining_sync_blocks = 0;
-                    self.quality.total_sync_blocks = 0;
-                    self.fail();
-                } else if self.quality.remaining_sync_blocks > 0 {
-                    trace!(
-                        "Was expecting {} more sync blocks from {}",
-                        self.quality.remaining_sync_blocks,
-                        self.address,
-                    );
-                    self.quality.remaining_sync_blocks = 0;
-                    self.quality.total_sync_blocks = 0;
-                }
+                self.cancel_sync();
                 Ok(PeerResponse::None)
                 //todo: should we notify the peer we are no longer expecting anything from them?
             }
             PeerAction::GotSyncBlock => {
-                if self.quality.remaining_sync_blocks > 0 {
-                    self.quality.remaining_sync_blocks -= 1;
-                } else {
-                    trace!("received unexpected or late sync block from {}", self.address);
-                }
+                self.register_received_sync_block();
                 Ok(PeerResponse::None)
             }
             PeerAction::ExpectingSyncBlocks(amount) => {
-                self.quality.remaining_sync_blocks += amount;
-                self.quality.total_sync_blocks += amount;
+                self.increment_sync_expectations(amount);
                 Ok(PeerResponse::None)
             }
             PeerAction::SoftFail => {
