@@ -20,12 +20,13 @@ use std::{
     time::{Duration, Instant},
 };
 
-use rand::{prelude::SliceRandom, rngs::SmallRng, seq::IteratorRandom, SeedableRng};
+use itertools::Itertools;
+use rand::{prelude::SliceRandom, rngs::SmallRng, SeedableRng};
 use tokio::task;
 
 use snarkos_metrics::{self as metrics, connections::*};
 
-use crate::{message::*, KnownNetworkMessage, NetworkError, Node, NodeType};
+use crate::{message::*, KnownNetworkMessage, NetworkError, Node, NodeType, Peer};
 use anyhow::*;
 
 impl Node {
@@ -78,8 +79,6 @@ impl Node {
             let capacity = (CAPACITY_PERCENTAGE * max_peers as f64).floor() as u32;
 
             (
-                // Beacons and sync providers disconnect down to 80% of their max to leave capacity open for new
-                // connections...
                 active_peer_count.saturating_sub(capacity),
                 // ...and don't connect to any peers on their own once above `0` peers.
                 0,
@@ -115,10 +114,7 @@ impl Node {
 
         // Attempt to connect to a few random beacons if the node has no active
         // connections or if it's a beacon itself.
-        if self.peer_book.get_active_peer_count() == 0
-            || self.is_of_type(NodeType::Beacon)
-            || self.is_of_type(NodeType::SyncProvider)
-        {
+        if self.peer_book.get_active_peer_count() == 0 || self.is_of_type(NodeType::Beacon) {
             let random_beacons = self
                 .config
                 .beacons()
@@ -129,6 +125,19 @@ impl Node {
             self.connect_to_addresses(&random_beacons).await;
         }
 
+        // Connect to a few random sync providers if the node is itself a sync provider, this will
+        // help to keep chain state in sync.
+        if self.is_of_type(NodeType::SyncProvider) {
+            let random_sync_providers = self
+                .config
+                .sync_providers()
+                .choose_multiple(&mut SmallRng::from_entropy(), 2)
+                .copied()
+                .collect::<Vec<_>>();
+
+            self.connect_to_addresses(&random_sync_providers).await;
+        }
+
         if number_to_connect != 0 {
             self.connect_to_disconnected_peers(number_to_connect as usize).await;
         }
@@ -143,6 +152,11 @@ impl Node {
                 error!("failed to broadcast pings: {:?}", e);
             }
         }
+
+        let peers = self.peer_book.serialize().await;
+        if let Err(e) = self.storage.store_peers(peers).await {
+            error!("failed to store peers to database: {:?}", e);
+        }
     }
 
     async fn initiate_connection(&self, remote_address: SocketAddr) -> Result<(), NetworkError> {
@@ -168,7 +182,11 @@ impl Node {
 
         metrics::increment_counter!(ALL_INITIATED);
 
-        self.peer_book.get_or_connect(self.clone(), remote_address).await?;
+        let stored_peer = self.storage.lookup_peers(vec![remote_address]).await?.remove(0);
+
+        self.peer_book
+            .get_or_connect(self.clone(), remote_address, stored_peer.as_ref())
+            .await?;
 
         Ok(())
     }
@@ -215,10 +233,10 @@ impl Node {
         // Local address must be known by now.
         let own_address = self.expect_local_addr();
 
-        let random_peers = {
+        let random_peers: Vec<SocketAddr> = {
             trace!(
                 "Connecting to {} disconnected peers",
-                cmp::min(count, self.peer_book.disconnected_peers().len())
+                cmp::min(count, self.peer_book.get_disconnected_peer_count() as usize)
             );
 
             // Obtain the collection of disconnected peers.
@@ -233,14 +251,29 @@ impl Node {
                 candidates.sort_unstable_by_key(|peer| peer.quality.last_connected);
             }
 
-            let addr_iter = candidates.iter().map(|peer| peer.address);
-
             if !self.is_of_type(NodeType::Client) {
-                addr_iter.take(count).collect()
+                candidates.into_iter().take(count).collect()
             } else {
-                addr_iter.choose_multiple(&mut SmallRng::from_entropy(), count)
+                // Floored if count is odd.
+                let random_count = count / 2;
+                let random_picks: Vec<Peer> = candidates
+                    .choose_multiple(&mut SmallRng::from_entropy(), random_count)
+                    .cloned()
+                    .collect();
+
+                candidates.sort_unstable_by(|x, y| y.block_height.cmp(&x.block_height));
+
+                candidates.truncate(count - random_count);
+                candidates
+                    .into_iter()
+                    .chain(random_picks)
+                    .unique_by(|x| x.address)
+                    .collect::<Vec<Peer>>()
             }
-        };
+        }
+        .iter()
+        .map(|peer| peer.address)
+        .collect();
 
         for remote_address in random_peers {
             let node = self.clone();
@@ -314,13 +347,14 @@ impl Node {
     pub(crate) async fn send_peers(&self, remote_address: SocketAddr, time_received: Option<Instant>) {
         // Broadcast the sanitized list of connected peers back to the requesting peer.
 
-        use crate::Peer;
-
         let connected_peers = self.peer_book.connected_peers_snapshot().await;
 
-        let basic_filter =
-            |peer: &Peer| peer.address != remote_address && !self.config.beacons().contains(&peer.address);
-        let strict_filter = |peer: &Peer| basic_filter(peer) && peer.is_routable.unwrap_or(false);
+        let basic_filter = |peer: &Peer| peer.address != remote_address;
+        let strict_filter = |peer: &Peer| {
+            basic_filter(peer)
+                && peer.quality.connection_transient_fail_count == 0
+                && peer.quality.connection_attempt_count > 0
+        };
 
         // Strictly filter the connected peers by only including the routable addresses.
         let strictly_filtered_peers: Vec<SocketAddr> = connected_peers
@@ -331,9 +365,7 @@ impl Node {
 
         // Beacons apply less strict filtering rules if the set is empty by falling back on
         // connected peers that may or may not be routable...
-        let peers = if (self.is_of_type(NodeType::SyncProvider) || self.is_of_type(NodeType::Beacon))
-            && strictly_filtered_peers.is_empty()
-        {
+        let peers = if self.is_of_type(NodeType::Beacon) && strictly_filtered_peers.is_empty() {
             let filtered_peers: Vec<SocketAddr> = connected_peers
                 .iter()
                 .filter(|peer| basic_filter(peer))
@@ -387,7 +419,7 @@ impl Node {
             // Inform the peer book that we found a peer.
             // The peer book will determine if we have seen the peer before,
             // and include the peer if it is new.
-            self.peer_book.add_peer(*peer_address).await;
+            self.peer_book.add_peer(*peer_address, None).await;
         }
 
         if let Some(known_network) = self.known_network() {
