@@ -18,6 +18,7 @@ use std::{net::SocketAddr, time::Duration};
 
 use snarkos_metrics::wrapped_mpsc;
 use snarkos_storage::Digest;
+use tokio::time::Instant;
 
 use crate::{Node, Peer, SyncInbound};
 use anyhow::*;
@@ -43,16 +44,16 @@ impl SyncBase {
         let mut interesting_peers = vec![];
         for mut node in self.node.peer_book.connected_peers_snapshot().await {
             let judge_bad = node.judge_bad();
-            if !judge_bad && node.quality.block_height as usize > our_block_height + 1 {
+            if !judge_bad && node.block_height as usize > our_block_height + 1 {
                 interesting_peers.push(node);
             }
         }
-        interesting_peers.sort_by(|x, y| y.quality.block_height.cmp(&x.quality.block_height));
+        interesting_peers.sort_by(|x, y| y.block_height.cmp(&x.block_height));
 
         // trim nodes close to us if any are > 10 blocks ahead
         if let Some(i) = interesting_peers
             .iter()
-            .position(|x| x.quality.block_height as usize <= our_block_height + 10)
+            .position(|x| x.block_height as usize <= our_block_height + 10)
         {
             interesting_peers.truncate(i + 1);
         }
@@ -109,11 +110,12 @@ impl SyncBase {
         moving_timeout_sec: u64,
         mut handler: F,
     ) {
-        loop {
-            let timeout = tokio::time::sleep(Duration::from_secs(timeout_sec));
-            let extra_time = Duration::from_secs(moving_timeout_sec);
+        let must_end = Instant::now() + Duration::from_secs(timeout_sec);
+        let timeout = tokio::time::sleep_until(must_end);
+        let extra_time = Duration::from_secs(moving_timeout_sec);
+        tokio::pin!(timeout);
 
-            tokio::pin!(timeout);
+        loop {
             tokio::select! {
                 biased;
 
@@ -127,8 +129,8 @@ impl SyncBase {
                     if handler(msg.unwrap()) {
                         break;
                     }
-                    let updated_timeout = timeout.deadline() + extra_time;
-                    timeout.as_mut().reset(updated_timeout);
+                    let new_timeout = (Instant::now() + extra_time).min(must_end);
+                    timeout.as_mut().reset(new_timeout);
                 },
             }
         }
