@@ -20,7 +20,7 @@ use snarkvm_dpc::testnet1::instantiated::Testnet1Transaction;
 use snarkvm_utilities::bytes::{FromBytes, ToBytes};
 
 use anyhow::*;
-use std::net::SocketAddr;
+use std::{net::SocketAddr, time::Instant};
 
 impl Node {
     ///
@@ -30,7 +30,7 @@ impl Node {
         if let Some(sync_node) = sync_node {
             info!("Updating memory pool from {}", sync_node);
 
-            self.peer_book.send_to(sync_node, Payload::GetMemoryPool).await;
+            self.peer_book.send_to(sync_node, Payload::GetMemoryPool, None).await;
         } else {
             debug!("No sync node is registered, memory pool could not be synced");
         }
@@ -46,13 +46,13 @@ impl Node {
     ) {
         debug!("Propagating a memory pool transaction to connected peers");
 
-        let local_address = self.local_address().unwrap();
+        let local_addr = self.expect_local_addr();
 
         for remote_address in self.connected_peers() {
-            if remote_address != transaction_sender && remote_address != local_address {
+            if remote_address != transaction_sender && remote_address != local_addr {
                 // Send a `Transaction` message to the connected peer.
                 self.peer_book
-                    .send_to(remote_address, Payload::Transaction(transaction_bytes.clone()))
+                    .send_to(remote_address, Payload::Transaction(transaction_bytes.clone()), None)
                     .await;
             }
         }
@@ -67,20 +67,24 @@ impl Node {
         source: SocketAddr,
         transaction: Vec<u8>,
     ) -> Result<()> {
-        if let Ok(tx) = Testnet1Transaction::read_le(&*transaction) {
-            let inserted = self.expect_sync().consensus.receive_transaction(tx.serialize()?).await;
+        let tx = Testnet1Transaction::read_le(&*transaction)?;
 
-            if inserted {
-                info!("Transaction added to memory pool.");
-                self.propagate_memory_pool_transaction(transaction, source).await;
-            }
+        let inserted = self.expect_sync().consensus.receive_transaction(tx.serialize()?).await;
+
+        if inserted {
+            info!("Transaction added to memory pool.");
+            self.propagate_memory_pool_transaction(transaction, source).await;
         }
 
         Ok(())
     }
 
     /// A peer has requested our memory pool transactions.
-    pub(crate) async fn received_get_memory_pool(&self, remote_address: SocketAddr) -> Result<()> {
+    pub(crate) async fn received_get_memory_pool(
+        &self,
+        remote_address: SocketAddr,
+        time_received: Option<Instant>,
+    ) -> Result<()> {
         let transactions = self
             .expect_sync()
             .consensus
@@ -97,7 +101,7 @@ impl Node {
         if !transactions.is_empty() {
             // Send a `MemoryPool` message to the connected peer.
             self.peer_book
-                .send_to(remote_address, Payload::MemoryPool(transactions))
+                .send_to(remote_address, Payload::MemoryPool(transactions), time_received)
                 .await;
         }
         Ok(())
